<?php

use GuzzleHttp\Exception\RequestException;

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2022 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
class SilkPayClass extends ObjectModel
{
    /**
     * @var string 订单
     */
    public $transactionid;
    /**
     * @var float 订单金额
     */
    public $order_amount;
    /**
     * @var int 财物车ID
     */
    public $id_cart;
    /**
     * @var string 支付方式 alipay,wechat
     */
    public $type;

    public $is_refund = 0;

    public $refund_amount = 0.000000;

    public $date_add;
    public static $definition = [
        'table' => 'silkpay',
        'primary' => 'id_silkpay',
        'fields' => [
            'transactionid' => ['type' => self::TYPE_STRING, 'validate' => 'isString'],
            'order_amount' => ['type' => self::TYPE_FLOAT, 'validate' => 'isPrice'],
            'id_cart' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'],
            'type' => ['type' => self::TYPE_STRING, 'validate' => 'isString'],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate'],
            'is_refund' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'refund_amount' => ['type' => self::TYPE_FLOAT, 'validate' => 'isPrice'],
        ]
    ];

    private $uid;
    private $sid;
    private $key_id;
    private $timestamp;
    private $cid;

    private $gateway_dev = 'https://extranet.silkpay.fr';
    private $gateway = 'https://extranet.silkpay.fr';

    private $alipay_online = '/silkpay/app/alipayonline';
    private $alipay_qrcode = '/silkpay/app/alipay/createcodedynamicweb';
//    private $alipay_qrcode = '/silkpay/app/alipayplus/code/online';

    private $wechat_qrcode = '/silkpay/app/wechat/nativepayweb';
    private $alipay_wap = '/silkpay/app/alipay/wap';
//    private $alipay_query = '/silkpay/app/alipayplus/orderquery/web';
    private $alipay_query = '/silkpay/app/alipay/orderquery';
    private $alipay_refund = '/silkpay/app/alipay/refundweb';

    private $wechat_query = '/silkpay/app/wechat/orderqueryweb';
    private $wechat_h5 = '/silkpay/app/wechat/h5';
    private $wechat_wap = '/silkpay/app/wechat/wap';
    private $wechat_jsapi = '/silkpay/app/wechat/jsapi';
    private $wechat_refund = '/silkpay/app/wechat/refundweb';

    private $unionpay = '/silkpay/app/card/unionpay';
    private $visa = '/silkpay/app/card/webpay';
    private $card_query = '/silkpay/app/card/orderquery';
    private $card_refund = '/silkpay/app/card/refund';

    private $yunshanfu_query = '/silkpay/app/yunshanfu/orderquery';
    private $yunshanfu_refund = '/silkpay/app/yunshanfu/refund';

    public function __construct()
    {
        $this->context = Context::getContext();
    }

    private function getGateway()
    {
        $this->cid = uniqid();
        $this->timestamp = (int)time();
        if (Configuration::get('SILKPAY_ENABLED_SANDBOX')) {
            $this->uid = 215;//215
            $this->sid = 203;//203
            $this->key_id = 'bf974e6841cb9715098825ac651b52df';
            return 'https://sandbox.silkpay.fr';
        } else {
            $this->uid = (int)Configuration::get('SILKPAY_ACCOUNT_UID');
            $this->sid = (int)Configuration::get('SILKPAY_ACCOUNT_SID');
            $this->key_id = Configuration::get('SILKPAY_ACCOUNT_KEY_ID');
            return 'https://extranet.silkpay.fr';
        }
    }

    public static function setCurrentStatusToPayment($id_cart)
    {
        file_put_contents(dirname(__FILE__) . '/order.txt', Configuration::get('PS_OS_PAYMENT'));
        return Db::getInstance()->update('orders', [
            'current_state' => (int)Configuration::get('PS_OS_PAYMENT')
        ], 'id_cart=' . (int)$id_cart);
    }

    /**
     * @param $content Context
     * @param $order Order
     * @param $currency Currency
     * @return array
     */
    public function getRequestParams($content, $order, $currency)
    {
        //收货人地址
        $address = new Address($order->id_address_invoice);

        $amount = Tools::convertPriceFull($order->total_paid, new Currency($order->id_currency), $currency);
        if ($amount <= 0) {
            $amount = '0.01';
        }
        return [
            'amount' => sprintf('%.' . $currency->precision . 'f', $amount),
            'currency' => $currency->iso_code,
            'lastname' => $content->customer->lastname,
            'firstname' => $content->customer->firstname,
            'ip' => Tools::getRemoteAddr(),
            'mail' => $content->customer->email,
            'phone' => !empty($address->phone_mobile) ? $address->phone_mobile : $address->phone,
            'street' => $address->address1 . $address->address2,
            'postal_code' => $address->postcode,
            'city' => $address->city,
            'country' => Country::getIsoById($address->id_country),
            'url_ok' => $this->context->link->getModuleLink('silkpay', 'confirmation')
        ];
    }

    /**
     * 取支付宝网关
     * @return string
     */
    private function getAlipayGateway($wap = false)
    {
        //使用支付宝+
        if ((bool)Configuration::get('SILKPAY_USE_ALIPAY_PLUS') === true) {
            if ($wap) {
                return '/silkpay/app/alipayplus/h5';
            }
            return '/silkpay/app/alipayplus/code/online';
        } else {
            if ($wap) {
                return '/silkpay/app/alipay/wap';
            }
            return '/silkpay/app/alipay/createcodedynamicweb';
        }
    }

    public function getAlipayQrcode(array $params = [])
    {
        $data = [
            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify')
        ];
        return $this->execute($this->getGateway() . $this->getAlipayGateway(), array_merge($params, $data));
    }

    public function getAlipayWapPay(array $params)
    {
        $data = [
            'url_ok' => $this->context->link->getModuleLink('silkpay', 'confirmation'),
            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify')
        ];
        return $this->execute($this->getGateway() . $this->getAlipayGateway(true), array_merge($params, $data));
    }

    public function alipayRefundOrder($transactionid)
    {
        $params = [
            'transactionid' => $transactionid
        ];
        $alipay_refund = '/silkpay/app/alipay/refundweb';
        if ((bool)Configuration::get('SILKPAY_USE_ALIPAY_PLUS') === true) {
            $alipay_refund = '/silkpay/app/alipayplus/refund/web';
        }
        return $this->execute($this->getGateway() . $alipay_refund, $params);
    }

    public function wechatRefundOrder($transactionid)
    {
        $params = [
            'transactionid' => $transactionid
        ];
        return $this->execute($this->getGateway() . $this->wechat_refund, $params);
    }

    public function cardRefundOrder($transactionid, $amount, $type)
    {
        $params = [
            'transactionid' => $transactionid,
            'amount' => $amount
        ];
        if ($type === 'union') {
            return $this->execute($this->getGateway() . $this->yunshanfu_refund, $params);
        } else {
            return $this->execute($this->getGateway() . $this->card_refund, $params);
        }
    }

    public function getWechatQrcode(array $params)
    {
//        $data = [
//            'ip' => Tools::getRemoteAddr(),
//            'url_ok' => $this->context->link->getModuleLink('silkpay', 'confirmation'),
//            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify')
//        ];
        return $this->execute($this->getGateway() . $this->wechat_qrcode, $params);
    }

    public function getWechatH5Pay(array $params)
    {
        $data = [
            'ip' => Tools::getRemoteAddr(),
            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify'),
            'wap_name' => $this->context->shop->name,
            'wap_url' => $this->context->shop->getBaseURL(true)
        ];
        return $this->execute($this->getGateway() . $this->wechat_h5, array_merge($params, $data));
    }

    public function getWechatJsApi(array $params)
    {
        $data = [
            'ip' => Tools::getRemoteAddr(),
            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify')
        ];
        return $this->execute($this->getGateway() . $this->wechat_jsapi, array_merge($params, $data));
    }

    public function alipayQueryOrder(string $transactionid)
    {
        $params = [
            'transaction_number' => $transactionid
        ];
        $alipay_refund = '/silkpay/app/alipay/orderquery';
        if ((bool)Configuration::get('SILKPAY_USE_ALIPAY_PLUS') === true) {
            $alipay_refund = '/silkpay/app/alipayplus/orderquery/web';
        }
        return $this->execute($this->getGateway() . $alipay_refund, $params);
    }

    public function wechatQueryOrder($transactionid)
    {
        $params = [
            'transaction_number' => $transactionid
        ];
        return $this->execute($this->getGateway() . $this->wechat_query, $params);
    }

    public function getUnionpay(array $params)
    {
        $data = [
            'cancel_url' => $this->context->link->getPageLink('index'),
        ];
        return $this->execute($this->getGateway() . $this->unionpay, array_merge($params, $data));
    }

    public function cardQueryOrder($transactionid, $type)
    {
        $params = [
            'transaction_number' => $transactionid
        ];
        if ($type === 'union') {
            return $this->execute($this->getGateway() . $this->yunshanfu_query, $params);
        } else {
            return $this->execute($this->getGateway() . $this->card_query, $params);
        }
    }

    public function getVisaPay(array $params)
    {
        $data = [
            'notify_url' => $this->context->link->getModuleLink('silkpay', 'notify'),
            'cancel_url' => $this->context->link->getPageLink('index'),
        ];
        return $this->execute($this->getGateway() . $this->visa, array_merge($params, $data));
    }

    private function getSignature()
    {
        return sha1($this->sid . '|' . $this->cid . '|' . $this->timestamp . '|' . $this->key_id);
    }

    /**
     * @param $url
     * @param $data
     * @return array|mixed
     */
    private function execute($url, $data)
    {
        $data['signature'] = $this->getSignature();
        $data['uid'] = $this->uid;
        $data['sid'] = $this->sid;
        $data['timestamp'] = $this->timestamp;
        $data['cid'] = $this->cid;
        $client = new \GuzzleHttp\Client([
            'verify' => false
        ]);
        try {
            if (version_compare(_PS_VERSION_, '8.0.0', '<')) {
                $response = $client->post($url, [
                    'body' => $data
                ]);
            } else {
                $response = $client->post($url, [
                    'form_params' => $data,
                    'headers' => [
                        'Content-Type' => 'application/x-www-form-urlencoded'
                    ]
                ]);
            }
            if ($response->getStatusCode() === 200) {
                $contentJson = $response->getBody()->getContents();
                $content = json_decode($contentJson, true);
                if (isset($content['response_code']) && $content['response_code'] === 'KO') {
                    PrestaShopLogger::addLog($contentJson, 3, 400, 'Silkpay');
                }
                return $content;
            } else {
                PrestaShopLogger::addLog('error for execute silkpay', 3, 400, 'Silkpay');
                return ['response_code' => 'KO', 'message' => 'Network error'];
            }
        } catch (RequestException|Exception $e) {
            PrestaShopLogger::addLog($e->getMessage(), 3, 400, 'Silkpay');
            return ['response_code' => 'KO', 'message' => 'Network error'];
        }
    }

    /**
     * @param bool $transactionid
     * @return array|bool|object|null
     */
    public function findRowByTransactionId(string $transactionid)
    {
        $query = new DbQuery();
        $query->select('*')
            ->from(self::$definition['table'])
            ->where('transactionid=\'' . pSQL($transactionid) . '\'');
        return Db::getInstance()->getRow($query);
    }

    public function saveOrder(array $data)
    {
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    public function updateOrder(array $data, int $id_silkpay)
    {
        return Db::getInstance()->update(self::$definition['table'], $data, 'id_silkpay=' . (int)$id_silkpay);
    }
///////////////////////微信支付需要使用///////////////////////////////////

    /**
     * 通过跳转获取用户的openid，跳转流程如下：
     * 1、设置自己需要调回的url及其其他参数，跳转到微信服务器https://open.weixin.qq.com/connect/oauth2/authorize
     * 2、微信服务处理完成之后会跳转回用户redirect_uri地址，此时会带上一些参数，如：code
     * @return mixed
     */
    public function getOpenid()
    {
        //通过code获得openid
        if (!Tools::getIsset('code')) {
            //触发微信返回code码
            $baseUrl = urlencode('http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] .
                $_SERVER['QUERY_STRING']);
            $url = $this->createOauthUrlForCode($baseUrl);
            Tools::redirect($url);
            exit();
        } else {
            //获取code码，以获取openid
            $code = Tools::getValue('code');
            return $this->getAppAccessTokenByCode($code);
        }
    }

    private function createOauthUrlForCode($redirectUrl)
    {
        $urlObj = [];
        $urlObj["appid"] = Configuration::get('SILKPAY_WECHAT_APP_ID');
        $urlObj["redirect_uri"] = "$redirectUrl";
        $urlObj["response_type"] = "code";
        $urlObj["scope"] = "snsapi_base";
        $urlObj["state"] = uniqid() . "#wechat_redirect";
        $bizString = $this->toUrlParams($urlObj);
        return "https://open.weixin.qq.com/connect/oauth2/authorize?" . $bizString;
    }

    private function toUrlParams($urlObj)
    {
        $buff = "";
        foreach ($urlObj as $k => $v) {
            if ($k != "sign") {
                $buff .= $k . "=" . $v . "&";
            }
        }
        $buff = trim($buff, "&");
        return $buff;
    }

    public function getAppAccessTokenByCode(string $code)
    {
        $uri = 'https://api.weixin.qq.com/sns/oauth2/access_token?appid=' .
            Configuration::get('SILKPAY_WECHAT_APP_ID') .
            '&secret=' . Configuration::get('SILKPAY_WECHAT_SECRET_KEY') . '&code=' .
            $code . '&grant_type=authorization_code';
        $client = new \GuzzleHttp\Client();
        $response = $client->get($uri);
        if ($response->getStatusCode() === 200) {
            return json_decode($response->getBody()->getContents(), true);
        } else {
            return false;
        }
    }

    /**
     * @param $transaction_id
     */
    public function getRecordByTranslationId($transaction_id)
    {
        $query = new DbQuery();
        $query->select('*')
            ->from(self::$definition['table'])
            ->where('transactionid=\'' . pSQL($transaction_id) . '\'');
        return Db::getInstance()->getRow($query);
    }

    /**
     * @param array $data
     * @param $transactionid
     * @return bool
     */
    public function updateRefund(array $data, $transactionid)
    {
        return Db::getInstance()->update(
            self::$definition['table'],
            $data,
            'transactionid=\'' . pSQL($transactionid) . '\''
        );
    }

    /**
     * 通过购物车ID取记录
     * @param int $id_order
     * @return false|string
     */
    public function getObjectIdByOrderId(int $id_order)
    {
        $query = new DbQuery();
        $query->select('id_silkpay')
            ->from('silkpay')
            ->where('id_cart=' . $id_order);
        return Db::getInstance()->getValue($query);
    }

    public function getObjectByOrderId(int $id)
    {
        $query = new DbQuery();
        $query->select('*')
            ->from(self::$definition['table'])
            ->where('id_cart=' . (int)$id);
        return Db::getInstance()->getRow($query);
    }
}
